@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * Linking this library statically or dynamically with other modules is
  * making a combined work based on this library.  Thus, the terms and
@@ -308,7 +308,8 @@ public class Request
     if (requestBodyWriter != null)
       {
         contentLength = requestBodyWriter.getContentLength();
-        if (contentLength > requestBodyNegotiationThreshold)
+        if (contentLength > requestBodyNegotiationThreshold &&
+                (connection.minorVersion > 0 || connection.majorVersion > 1))
           {
             expectingContinue = true;
             setHeader("Expect", "100-continue");
@@ -385,7 +386,7 @@ public class Request
                     retry = true;
                   }
               }
-            else if (sc == 100 && expectingContinue)
+            else if (sc == 100)
               {
                 requestHeaders.remove("Expect");
                 setHeader("Content-Length", Integer.toString(contentLength));
@@ -524,6 +525,7 @@ public class Request
             throw new ProtocolException("Unsupported Content-Encoding: " +
                                         contentCoding);
           }
+        response.headers.remove("Content-Encoding");
       }
     
     // Persistent connections are the default in HTTP/1.1
@@ -610,14 +612,14 @@ public class Request
       {
         Properties params = parseAuthParams(challenge.substring(si + 1));
         String realm = params.getProperty("realm");
-        String nonce = params.getProperty("nonce");
+        String nonceParam = params.getProperty("nonce");
         String qop = params.getProperty("qop");
         String algorithm = params.getProperty("algorithm");
         String digestUri = getRequestURI();
         Credentials creds = authenticator.getCredentials(realm, attempts);
         String username = creds.getUsername();
         String password = creds.getPassword();
-        connection.incrementNonce(nonce);
+        connection.incrementNonce(nonceParam);
         try
           {
             MessageDigest md5 = MessageDigest.getInstance("MD5");
@@ -637,7 +639,7 @@ public class Request
                 md5.reset();
                 md5.update(ha1);
                 md5.update(COLON);
-                md5.update(nonce.getBytes("US-ASCII"));
+                md5.update(nonceParam.getBytes("US-ASCII"));
                 md5.update(COLON);
                 md5.update(cnonce);
                 ha1 = md5.digest();
@@ -662,10 +664,10 @@ public class Request
             md5.reset();
             md5.update(ha1Hex.getBytes("US-ASCII"));
             md5.update(COLON);
-            md5.update(nonce.getBytes("US-ASCII"));
+            md5.update(nonceParam.getBytes("US-ASCII"));
             if ("auth".equals(qop) || "auth-int".equals(qop))
               {
-                String nc = getNonceCount(nonce);
+                String nc = getNonceCount(nonceParam);
                 byte[] cnonce = generateNonce();
                 md5.update(COLON);
                 md5.update(nc.getBytes("US-ASCII"));
@@ -681,7 +683,7 @@ public class Request
             String authorization = scheme + 
               " username=\"" + username + "\"" +
               " realm=\"" + realm + "\"" +
-              " nonce=\"" + nonce + "\"" +
+              " nonce=\"" + nonceParam + "\"" +
               " uri=\"" + digestUri + "\"" +
               " response=\"" + digestResponse + "\"";
             setHeader("Authorization", authorization);
@@ -749,9 +751,9 @@ public class Request
    * Returns the number of times the specified nonce value has been seen.
    * This always returns an 8-byte 0-padded hexadecimal string.
    */
-  String getNonceCount(String nonce)
+  String getNonceCount(String nonceParam)
   {
-    int nc = connection.getNonceCount(nonce);
+    int nc = connection.getNonceCount(nonceParam);
     String hex = Integer.toHexString(nc);
     StringBuffer buf = new StringBuffer();
     for (int i = 8 - hex.length(); i > 0; i--)
@@ -813,11 +815,11 @@ public class Request
     String value = null;
     String comment = null;
     String domain = connection.getHostName();
-    String path = this.path;
-    int lsi = path.lastIndexOf('/');
+    String p = path;
+    int lsi = p.lastIndexOf('/');
     if (lsi != -1)
       {
-        path = path.substring(0, lsi);
+        p = p.substring(0, lsi);
       }
     boolean secure = false;
     Date expires = null;
@@ -858,7 +860,7 @@ public class Request
                   }
                 else if ("Path".equalsIgnoreCase(attr))
                   {
-                    path = val;
+                    p = val;
                   }
                 else if ("Secure".equalsIgnoreCase(val))
                   {
@@ -894,7 +896,7 @@ public class Request
                 if (i == len || c == ',')
                   {
                     Cookie cookie = new Cookie(name, value, comment, domain,
-                                               path, secure, expires);
+                                               p, secure, expires);
                     cookieManager.setCookie(cookie);
                   }
                 if (c == ',')
@@ -904,10 +906,10 @@ public class Request
                     value = null;
                     comment = null;
                     domain = connection.getHostName();
-                    path = this.path;
+                    p = path;
                     if (lsi != -1)
                       {
-                        path = path.substring(0, lsi);
+                        p = p.substring(0, lsi);
                       }
                     secure = false;
                     expires = null;
