@@ -16,7 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  * Linking this library statically or dynamically with other modules is
  * making a combined work based on this library.  Thus, the terms and
@@ -38,15 +38,24 @@
 
 package gnu.inet.nntp;
 
+import gnu.inet.util.CRLFInputStream;
+import gnu.inet.util.CRLFOutputStream;
+import gnu.inet.util.EmptyX509TrustManager;
+import gnu.inet.util.LineInputStream;
+import gnu.inet.util.SaslCallbackHandler;
+import gnu.inet.util.SaslInputStream;
+import gnu.inet.util.SaslOutputStream;
+import gnu.inet.util.TraceLevel;
+
 import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
-import java.io.InputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
-import java.lang.reflect.Constructor;
 import java.net.InetSocketAddress;
 import java.net.ProtocolException;
 import java.net.Socket;
+import java.security.GeneralSecurityException;
 import java.text.DateFormat;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
@@ -58,36 +67,26 @@ import java.util.TimeZone;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import javax.net.ssl.SSLContext;
+import javax.net.ssl.SSLSocket;
+import javax.net.ssl.SSLSocketFactory;
+import javax.net.ssl.TrustManager;
 import javax.security.auth.callback.CallbackHandler;
 import javax.security.sasl.Sasl;
 import javax.security.sasl.SaslClient;
-import javax.security.sasl.SaslException;
-
-import gnu.inet.util.CRLFInputStream;
-import gnu.inet.util.CRLFOutputStream;
-import gnu.inet.util.LineInputStream;
-import gnu.inet.util.MessageInputStream;
-import gnu.inet.util.SaslCallbackHandler;
-import gnu.inet.util.SaslInputStream;
-import gnu.inet.util.SaslOutputStream;
-import gnu.inet.util.TraceLevel;
 
 /**
  * An NNTP client.
  * This object is used to establish and manage a connection to an NNTP
  * server.
  *
- * @author <a hef='mailto:dog@gnu.org'>Chris Burdess</a>
+ * @author <a href='mailto:dog@gnu.org'>Chris Burdess</a>
+ * @author <a href='mailto:jan.michalica@centire.com'>Jan Michalica</a>
  */
 public class NNTPConnection
   implements NNTPConstants
 {
 
-  /**
-   * The logger used for NNTP protocol traces.
-   */
-  public static final Logger logger = Logger.getLogger("gnu.inet.nntp");
-
   /**
    * The network trace level.
    */
@@ -97,6 +96,16 @@ public class NNTPConnection
    * The default NNTP port.
    */
   public static final int DEFAULT_PORT = 119;
+  
+  /**
+   * The default NNTPS port
+   */
+  public static final int DEFAULT_SSL_PORT = 563;
+
+  /**
+   * The logger used for NNTP protocol traces.
+   */
+  public final Logger logger = Logger.getLogger("gnu.inet.nntp");
 
   /**
    * The hostname of the host we are connected to.
@@ -141,6 +150,7 @@ public class NNTPConnection
   private static final String DOT = ".";
   private static final String US_ASCII = "US-ASCII";
 
+
   /**
    * Creates a new connection object.
    * @param hostname the hostname or IP address of the news server
@@ -148,7 +158,7 @@ public class NNTPConnection
   public NNTPConnection(String hostname)
     throws IOException
   {
-    this(hostname, DEFAULT_PORT, 0, 0);
+    this(hostname, -1, 0, 0, false, null, true);
   }
 
   /**
@@ -159,9 +169,22 @@ public class NNTPConnection
   public NNTPConnection(String hostname, int port)
     throws IOException
   {
-    this(hostname, port, 0, 0);
+    this(hostname, port, 0, 0, false, null, true);
   }
-
+  
+  /**
+   * Creates a new secure connection using the specified trust manager.
+   * @param host the name of the host to connect to
+   * @param port the port to connect to, or -1 for the default
+   * @param tm a trust manager used to check SSL certificates, or null to
+   * use the default
+   */
+  public NNTPConnection(String host, int port, TrustManager tm)
+    throws IOException
+  {
+    this(host, port, 0, 0, true, tm, true);
+  }
+  
   /**
    * Creates a new connection object.
    * @param hostname the hostname or IP address of the news server
@@ -172,37 +195,103 @@ public class NNTPConnection
   public NNTPConnection(String hostname, int port,
                         int connectionTimeout, int timeout)
     throws IOException
+  {
+    this(hostname, port, connectionTimeout, timeout, false, null, true);
+  }
+
+  /**
+   * Creates a new connection object.
+   * @param hostname the hostname or IP address of the news server
+   * @param port the port to connect to
+   * @param connectionTimeout the socket connection timeout
+   * @param timeout the read timeout on the socket
+   * @param init initialise the connection
+   */
+  public NNTPConnection(String hostname, int port,
+                        int connectionTimeout, int timeout,
+                        boolean secure, TrustManager tm,
+                        boolean init)
+    throws IOException
   {
     if (port < 0)
       {
-        port = DEFAULT_PORT;
+    	port = secure ? DEFAULT_SSL_PORT : DEFAULT_PORT;
       }
     
     this.hostname = hostname;
     this.port = port;
     
     // Set up the socket and streams
-    socket = new Socket();
-    InetSocketAddress address = new InetSocketAddress(hostname, port);
-    if (connectionTimeout > 0)
+    try
       {
-        socket.connect(address, connectionTimeout);
+        socket = new Socket();
+        InetSocketAddress address = new InetSocketAddress(hostname, port);
+        if (connectionTimeout > 0)
+          {
+            socket.connect(address, connectionTimeout);
+          }
+        else
+          {
+            socket.connect(address);
+          }
+        if (timeout > 0)
+          {
+            socket.setSoTimeout(timeout);
+          }
+        if (secure)
+          {
+		    SSLSocketFactory factory = getSSLSocketFactory(tm);
+		    SSLSocket ss =
+			  (SSLSocket) factory.createSocket(socket, hostname, port, true);
+		    String[] protocols = { "TLSv1", "SSLv3" };
+		    ss.setEnabledProtocols(protocols);
+		    ss.setUseClientMode(true);
+		    ss.startHandshake();
+		    socket = ss;
+	      }
       }
-    else
+    catch (GeneralSecurityException e)
       {
-        socket.connect(address);
+    	throw (IOException) new IOException().initCause(e);
       }
-    if (timeout > 0)
-      {
-        socket.setSoTimeout(timeout);
-      }
-    InputStream in = socket.getInputStream();
-    in = new CRLFInputStream(in);
-    this.in = new LineInputStream(in);
-    OutputStream out = socket.getOutputStream();
-    out = new BufferedOutputStream(out);
-    this.out = new CRLFOutputStream(out);
+    InputStream is = socket.getInputStream();
+    is = new CRLFInputStream(is);
+    in = new LineInputStream(is);
+    OutputStream os = socket.getOutputStream();
+    os = new BufferedOutputStream(os);
+    out = new CRLFOutputStream(os);
     
+    if (init)
+      init();
+  }
+
+  /**
+   * Returns a configured SSLSocketFactory to use in creating new SSL
+   * sockets.
+   * @param tm an optional trust manager to use
+   */
+  protected SSLSocketFactory getSSLSocketFactory(TrustManager tm)
+    throws GeneralSecurityException
+  {
+    if (tm == null)
+      {
+        tm = new EmptyX509TrustManager();
+      }
+    SSLContext context = SSLContext.getInstance("TLS");
+    TrustManager[] trust = new TrustManager[] { tm };
+    context.init(null, trust, null);
+    return context.getSocketFactory();
+  }
+  
+  /**
+   * Initialises the connection.
+   * Unless the init parameter was provided with the value false,
+   * do not call this method. Otherwise call it only once after e.g.
+   * configuring logging.
+   */
+  public void init()
+    throws IOException
+  {
     // Read the welcome message(RFC977:2.4.3)
     StatusResponse response = parseResponse(read());
     switch (response.status)
@@ -217,6 +306,76 @@ public class NNTPConnection
       }
   }
   
+  /**
+   * Negotiate TLS over the current connection.
+   * This depends on many features, such as the JSSE classes being in the
+   * classpath. Returns true if successful, false otherwise.
+   */
+  public boolean starttls()
+    throws IOException
+  {
+    return starttls(new EmptyX509TrustManager());
+  }
+  
+  /**
+   * This command performs a TLS negotiation.
+   * See RFC 4642 for details.
+   * @param tm the custom trust manager to use
+   * @return true if successful, false otherwise
+   */
+  public boolean starttls(TrustManager tm)
+  	throws IOException
+  {
+	  try
+	    {
+		  SSLSocketFactory factory = getSSLSocketFactory(tm);
+		  
+		  send(STARTTLS);
+		  StatusResponse response = parseResponse(read());
+		  
+		  switch (response.status)
+		    {
+		  case TLS_INIT_ERROR:
+		  case PERMISSION_DENIED: // e.g. TLS is already active
+		  case COMMAND_NOT_RECOGNIZED: // i.e. server does not implement/allow STARTTLS
+		  /*
+		   * Although RFC 4642 forbids it,
+		   * INN returns this when STARTTLS has already succeeded.
+		   */
+		  case ENCRYPTION_OR_AUTH_REQUIRED:
+		  	  return false;
+		  case CONTINUE_TLS_NEGOTIATION:
+			  break;
+		  default:
+			  throw new NNTPException(response);
+		    }
+		  
+		  SSLSocket ss =
+	          (SSLSocket) factory.createSocket(socket, hostname, port, true);
+	      String[] protocols = { "TLSv1", "SSLv3" };
+	      ss.setEnabledProtocols(protocols);
+	      ss.setUseClientMode(true);
+	      ss.startHandshake();
+		  
+	      // Set up streams
+	      InputStream in = socket.getInputStream();
+	      in = new CRLFInputStream(in);
+	      this.in = new LineInputStream(in);
+	      OutputStream out = socket.getOutputStream();
+	      out = new BufferedOutputStream(out);
+	      this.out = new CRLFOutputStream(out);
+	      
+		  return true;
+		  
+	    }
+	  catch (GeneralSecurityException e)
+	    {
+		  e.printStackTrace();
+		  return false;
+	    }
+  }
+  
+  
   /**
    * Return the welcome message sent by the server in reply to the initial
    * connection.
@@ -380,8 +539,7 @@ public class NNTPConnection
       case HEAD_FOLLOWS:
       case BODY_FOLLOWS:
         ArticleResponse aresponse = (ArticleResponse) response;
-        ArticleStream astream =
-          new ArticleStream(new MessageInputStream(in));
+        ArticleStream astream = new ArticleStream(in);
         pendingData = astream;
         aresponse.in = astream;
         return aresponse;
@@ -1158,7 +1316,7 @@ public class NNTPConnection
     p.put("gnu.crypto.sasl.username", username);
     p.put("gnu.crypto.sasl.password", password);
     SaslClient sasl =
-      Sasl.createSaslClient(m, null, "smtp",
+      Sasl.createSaslClient(m, null, "nntp",
                              socket.getInetAddress().getHostName(),
                              p, ch);
     if (sasl == null)
@@ -1184,15 +1342,15 @@ public class NNTPConnection
         if ("auth-int".equalsIgnoreCase(qop)
             || "auth-conf".equalsIgnoreCase(qop))
           {
-            InputStream in = socket.getInputStream();
-            in = new BufferedInputStream(in);
-            in = new SaslInputStream(sasl, in);
-            in = new CRLFInputStream(in);
-            this.in = new LineInputStream(in);
-            OutputStream out = socket.getOutputStream();
-            out = new BufferedOutputStream(out);
-            out = new SaslOutputStream(sasl, out);
-            this.out = new CRLFOutputStream(out);
+            InputStream is = socket.getInputStream();
+            is = new BufferedInputStream(is);
+            is = new SaslInputStream(sasl, is);
+            is = new CRLFInputStream(is);
+            in = new LineInputStream(is);
+            OutputStream os = socket.getOutputStream();
+            os = new BufferedOutputStream(os);
+            os = new SaslOutputStream(sasl, os);
+            out = new CRLFOutputStream(os);
           }
         return true;
       case PERMISSION_DENIED:
